@@ -27,7 +27,7 @@
 #include "gdk-pixbuf-private.h"
 #include "gdk-pixbuf-io.h"
 
-#define PCX_DEBUG
+#undef PCX_DEBUG
 
 #define PCX_TASK_DONE 0
 #define PCX_TASK_LOAD_HEADER 1
@@ -353,7 +353,7 @@ pcx_increment_load_data_1(struct pcx_context *context)
 		}
 
 		if(context->updated_func)
-			context->updated_func(context->pixbuf, 0, 0, context->width, context->height, context->user_data);
+			context->updated_func(context->pixbuf, 0, context->current_line, context->width, 1, context->user_data);
 
 		context->current_line++;
 
@@ -493,7 +493,7 @@ pcx_load_palette_8(struct pcx_context *context)
 				}
 
 				if(context->updated_func)
-					context->updated_func(context->pixbuf, 0, context->current_line, context->width, 1, context->user_data);
+					context->updated_func(context->pixbuf, 0, i, context->width, 1, context->user_data);
 			}
 
 #ifdef PCX_DEBUG
@@ -758,6 +758,6 @@ MODULE_ENTRY (pcx, fill_info) (GdkPixbufFormat *info)
 	info->description = N_("The PCX image format");
 	info->mime_types = mime_types;
 	info->extensions = extensions;
-	info->flags = 0;
+	info->flags = GDK_PIXBUF_FORMAT_THREADSAFE;
 	info->license = "LGPL";
 }
