@@ -472,7 +472,7 @@ _gtk_rbtree_free (GtkRBTree *tree)
   if (tree->parent_node &&
       tree->parent_node->children == tree)
     tree->parent_node->children = NULL;
-  _gtk_rbnode_free (tree->nil);
+  g_free (tree->nil);
   g_free (tree);
 }
 
@@ -895,7 +895,8 @@ _gtk_rbtree_mark_invalid (GtkRBTree *tree)
 
 void
 _gtk_rbtree_set_fixed_height (GtkRBTree *tree,
-			      gint       height)
+			      gint       height,
+			      gboolean   mark_valid)
 {
   GtkRBNode *node;
 
@@ -911,10 +912,14 @@ _gtk_rbtree_set_fixed_height (GtkRBTree *tree,
   do
     {
       if (GTK_RBNODE_FLAG_SET (node, GTK_RBNODE_INVALID))
-	_gtk_rbtree_node_set_height (tree, node, height);
+        {
+	  _gtk_rbtree_node_set_height (tree, node, height);
+	  if (mark_valid)
+	    _gtk_rbtree_node_mark_valid (tree, node);
+	}
 
       if (node->children)
-	_gtk_rbtree_set_fixed_height (node->children, height);
+	_gtk_rbtree_set_fixed_height (node->children, height, mark_valid);
     }
   while ((node = _gtk_rbtree_next (tree, node)) != NULL);
 }
